/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _ENTITY_H_
#define _ENTITY_H_

#include <set>
#include <map>

#include "mesh.h"
#include "material.h"



class Entity
{
public:
	Entity();
    Entity(Material *material, Mesh *mesh, bool castShadow);	
	virtual ~Entity();
		
	Material* getMaterial(void);
	void setMaterial(Material* material);

	Mesh* getMesh();
	void setMesh(Mesh* mesh);
		
	void addMaterialsInTheSet(std::set<Material*> &matSet);
	void addMeshesInTheSet(std::set<std::pair<Mesh*, int> > &meshSet, int nbTextures);
	
		
	void setVisibility(bool visible);
	bool isVisible();

	void setNumClick(int numClick);
	int getNumClick();	

	bool castShadow();		
		
		
	void initialize(Material *fatherMaterial);
	std::set<int> getMaterialKeys();
		
	virtual void applyTransformation() = 0;		
		
	void render(int lightKey, shaderType shader, Color groundColor, Color skyColor, Vector3 hemiAxis);
	void drawGeometry(int nbTextures);		
	void renderOccluders();
	void renderClickableObjects();

	bool isAChild(int key);

	void addChild(Entity *child);	
	void addChild(Entity *child, int key);
	void removeChild(int key);
	void removeAllChildren();		
		
protected:		
	Mesh *mpMesh;	
    Material* mpMaterial;
				
	bool mCastShadow;
	bool mVisible;
		
	int mNumClick;	/*-2 if no son is clickable
					  -1 if the entity is not clickable, but one of the sons can be clickable
		 			  >0 if the entity is clickable*/
		
	std::map<int, Entity*> children;	
};



class EntityWithPosition: public virtual Entity {
public:
	EntityWithPosition();
	EntityWithPosition(Vector3 center, Vector3 rotAxis, float angle);
	EntityWithPosition(Material *material, Mesh *mesh, bool castShadow, Vector3 center, Vector3 rotAxis, float angle);
	~EntityWithPosition();

	void setPosition(Vector3 center);		
	void setPosition(Vector3 center, Vector3 rotAxis, float angle);
	void applyTransformation();
	
protected:
	Vector3 mCenter;
	Vector3 mRotAxis;
	float mAngle;
};


#endif // _ENTITY_H_
