/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MATERIAL_H_
#define _MATERIAL_H_

#include "glUtil.h"
#include "textureLayer.h"
#include "color.h"

#include "graphicOptions.h"


class Material
{
public:

	Material(std::string name, int numColor);
	~Material();

	void setName(std::string name);
	std::string getName();
		
	int getNbEntities ();
		
	void setAmbientDiffuseColor(float coeff, float alpha);
	void setAmbientDiffuseColor(Color color);
	void setSpecularColor(float coeff, float shininess);
	void setSpecularColor(Color color);
	void setEmissionColor(float coeff);
	void setEmissionColor(Color color);		
		
	Color getSpecular();
	Color getAmbientDiffuse();
	Color getEmission();
		
	int getNbTexColor();
	int getNbTexNormal();
	int getNbTexGloss();
	int getNbTexEmi();
		
	int getMaterialKey();
		
	void setBlendFunc(GLenum blendSrcFunc, GLenum blendDstFunc);
	void setAlphaTest(float alphaTestValue);
		
	void activate(int lightKey, shaderType shader);
	void release();
		
	void addTexture(wstring text, int color, textureType type, int width, int height, GLenum wrapMode, bool mipmaps);
	void addTexture(wstring text, int textColor, float textColorCoeff, int backgroundColor, float backgroundCoeff, textureType type, int width, int height, GLenum wrapMode, bool mipmaps);
	
	void addTexture(std::string filename, bool smallTex, int color, textureType type, GLenum wrapMode, bool mipmaps, GraphicOptions *graphics);
	void addTexture(std::string filename, bool smallTex, int color, textureType type, GLenum wrapMode, float scaleS, float scaleT, bool mipmaps, GraphicOptions *graphics);
		
	int getNumberOfTextures();
		
	void incNbEntities();
	void decNbEntities();
		
		
protected:	
	bool testNbTextures(textureType type, GraphicOptions *graphics);
	void addTextureAndUpdateNbTex(ImagePng *image, textureType type, GLenum wrapMode, bool mipmaps, float scaleS, float scaleT);
	
	
	std::string mName;
	
	int mNumColor;
	
	Color mEmission;
	Color mAmbientDiffuse;
	Color mSpecular;
	
	std::vector<TextureLayer*> textures;
		
	int nbTexColor;
	int nbTexNormal;		
	int nbTexGloss;
	int nbTexEmission;
		
	
	//nbTexColor + nbTexNormal * 4 + nbTexGloss * 4*4 + nbTexEmission *4*4*4 
	int materialKey;
		
	bool mActiveBlending;
	GLenum mBlendSrcFunc;
	GLenum mBlendDstFunc;
		
	bool mAlphaTest;
	float mAlphaTestValue;
		
	int mNbEntities;	//number of Entity which uses this material
		
};



#endif // _MATERIAL_H_
