/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mirror.h"

Mirror::Mirror(Material *material, float dim1, int nb1, float dim2, int nb2, Vector3 center, Vector3 rotAxis, float angle): 
EntityWithPosition (center, rotAxis, angle) {
	
	RectangleMesh *rectangle = new RectangleMesh(dim1, nb1, dim2, nb2);
	setMesh (rectangle);
	setMaterial (material);
	if(material != NULL) {
		material->setBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	
	Vector3 zAxis = Vector3(0.,0.,1.);
	Vector3 normalMirror;	// rotation of the vector zAxis around the vector rotAxis by the angle theta
	float theta = angle * M_PI /180.;
	
	//Rodrigues' rotation formula
	normalMirror = cos(theta) * zAxis + sin(theta) * (rotAxis ^ zAxis) + (1. - cos(theta)) * (zAxis*rotAxis) * rotAxis;
	float d = - (normalMirror * center);
	
	equation[0] = normalMirror.x;
	equation[1] = normalMirror.y;
	equation[2] = normalMirror.z;
	equation[3] = d;
		
}

Mirror::~Mirror() {
}


void Mirror::getEquation(double eq[4]) {
	eq[0] = equation[0];
	eq[1] = equation[1];
	eq[2] = equation[2];
	eq[3] = equation[3];	
}


void Mirror::getReflectionMatrix(double matrix[16]) {
	matrix[0] = 1.-2.*equation[0]*equation[0];
	matrix[1] = -2.*equation[1]*equation[0];
	matrix[2] = -2.*equation[2]*equation[0];
	matrix[3] = 0.;
		
	matrix[4] = -2.*equation[0]*equation[1];
	matrix[5] = 1.-2.*equation[1]*equation[1];
	matrix[6] = -2.*equation[2]*equation[1];
	matrix[7] = 0.;
		
	matrix[8] = -2.*equation[0]*equation[2];
	matrix[9] = -2.*equation[1]*equation[2];
	matrix[10] = 1.-2.*equation[2]*equation[2];
	matrix[11] = 0.;
		
	matrix[12] = -2.*equation[0]*equation[3];
	matrix[13] = -2.*equation[1]*equation[3];
	matrix[14] = -2.*equation[2]*equation[3];
	matrix[15] = 1.;
}




