/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ticTacToe.h"


TicTacToe::TicTacToe(): Game("Tic-Tac-Toe", "TicTacToe") {	
	mKeyboardOptions = new TicTacToeKeyboardOptions(mDirPath + "/keyboardOptions.dat");	
	
	std::vector<int> defaultColors;
	defaultColors.push_back(14);
	defaultColors.push_back(2);
	defaultColors.push_back(10);
	options = new TicTacToeOptions(mDirPath + "/options.dat", defaultColors);
		
	distInitCam = (float) (options->getGridSize()) * GRID_INTERVAL * 1.5;
	distCam = distInitCam;	
	
	gameDisplayIndex = 0;
	scoreDisplayIndex = 1;	
	menuIndex = 0;
	
}

TicTacToe::~TicTacToe(){	
	delete options;	
}

void TicTacToe::initDisplayEnvironments () {
	Display3D *gameDisp3D = new Display3D();
	scene->addDisplay3D(gameDisp3D);

	float farClip = (float) (options->getGridSize()) * GRID_INTERVAL * 3.;
	
	scene->setClearColor(options->getColor(1),-0.95);	
	gameFog = new Fog(GL_LINEAR, 1., GRID_INTERVAL * 3. , farClip , options->getColor(1), -0.95);
	gameDisp3D->setFog(gameFog);

	int width = generalGraphics->getScreenWidth();
	int height = generalGraphics->getScreenHeight();	
	float fov = 70.;
	
	gameCamera = new Camera(distInitCam, fov, SPHERES_RADIUS, farClip, 0, 0, width*3/4, height);	
	gameDisp3D->setCamera(gameCamera);
	reinitPosCamera();
	

	Display3D *scoreDisp3D = new Display3D();
	scene->addDisplay3D(scoreDisp3D);
	
	scoreFog = new Fog(GL_LINEAR, 1., 130. , 200. , options->getColor(1), -0.95);
	scoreDisp3D->setFog(scoreFog);
	
	scoreCamera = new Camera(150., fov, 100., 200., width*3/4, 0, width/4, height);	
	scoreDisp3D->setCamera(scoreCamera);

	
	Display3D *menuDisp3D = new Display3D();
	scene->addGuiLayer(menuDisp3D);
	menuDisp3D->setFog(new Fog(GL_LINEAR, 1., 80. , 300. , options->getColor(0), -0.95));

	float angle = M_PI/12.;
	float distCam = 150.;
	menuCamera = new Camera(distCam, fov, 50., 200., 0, 0, width, height);
	menuCamera->setPositionLookingAtO(Vector3(0., distCam * sin(angle), distCam * cos(angle)));
	menuDisp3D->setCamera(menuCamera);	
}

void TicTacToe::createLights () {
	
	Display3D *gameDisp3D = scene->getDisplay3D(gameDisplayIndex);	
	
	float distSpot = (float) (options->getGridSize()) * GRID_INTERVAL * 2.;
	float spotD0 = (float) (options->getGridSize()) * GRID_INTERVAL * 4.;
	Vector3 spotPosition = Vector3(0., distSpot, 0.);
	Vector3 spotDirection = Vector3(0., -1.,0.);
	spot = new LightSpot(0, 2, spotPosition, spotDirection, 30., 40., 2., spotD0);
	gameDisp3D->addLightSpot(spot);

	
		if(options->areColoredLightsEnabled()) {	
		float distLights = (float) (options->getGridSize()) * GRID_INTERVAL * 1.5;
		float phi;
		LightPoint *pointLight;
		SphereWithPosition *sphereLight;
		Material *materialSphereLight;
		
		for(int i=0;i<options->getNbPlayers();i++) {
			phi = (float) i * 2.*M_PI / (float) (options->getNbPlayers()-1);
			Vector3 lightPosition = Vector3(distLights * sin(phi), 0., distLights * cos(phi));
			pointLight = new LightPoint(options->getColor(i+1), 0., 2, lightPosition, 0.3, (float) (options->getGridSize()) * GRID_INTERVAL / 3. );
			gameDisp3D->addLightPoint(pointLight);
			ligthsPoint.push_back(pointLight);
		}
	}

	Display3D *scoreDisp3D = scene->getDisplay3D(scoreDisplayIndex);
	spotDirection = Vector3(0., -cos(M_PI/8.), -sin(M_PI/8.));
	spotPosition = -200. * spotDirection;	
	spotScore = new LightSpot(0, 0.5, 2, spotPosition, spotDirection, 30., 40., 1.5, 200.);
	scoreDisp3D->addLightSpot(spotScore);
	
	
	Display3D *menuDisp3D  = scene->getGuiLayer(menuIndex);
	float angle = 0.;
	spotPosition = Vector3(0., 150.*sin(angle), 150.*cos(angle));
	spotDirection = -1.*spotPosition;
	spotMenu = new LightSpot(0, 0.5, 2, spotPosition, spotDirection, 30., 40., 1.5, 200.);				   
	menuDisp3D->addLightSpot(spotMenu);
	
	
}

void TicTacToe::createScore() {
	Display3D *scoreDisp3D = scene->getDisplay3D(scoreDisplayIndex);	
	int scoreMax = options->getVictoryCondition();
	if(scoreMax == -1) {
		scoreMax = 12;
	}
	score = new Score(options->getColors(), scoreMax, 24., 20, 4., 2., 6, 6., 12, 8, Vector3(0.,0.,0.));
	scoreDisp3D->addEntity(score); 
}


void TicTacToe::createMenu() {

	menu = new NewtMenu(L"Tic-Tac-Toe", Vector3(0.,0.,0.), Vector3(1., 0., 0.), 0.);
	
	std::vector<wstring> menuNames;
	menuNames.push_back(L"Continue");
	menuNames.push_back(L"New game");				
	menuNames.push_back(L"Quit");
	menu->setMenuItems(menuNames, 14, 0., 5, -0.95, 100., 20.);

	Display3D *menuDisp3D  = scene->getGuiLayer(menuIndex);
	menuDisp3D->addEntity(menu);
}




void TicTacToe::updateAnimations(double currentTime, double intervalTime) {
	if(spot->getStatus()) {
		float phi = currentTime/5.;
		float lambda = 1.2 - cos(currentTime/4.)/5.;
		float distSpot = (float) (options->getGridSize()) * GRID_INTERVAL * 2.;
		Vector3 spotDirection = Vector3(-cos(lambda)*cos(phi), -sin(lambda), -cos(lambda)*sin(phi));
		Vector3 spotPosition = -distSpot * spotDirection;
		spot->setSpotDirection(spotDirection);
		spot->setPosition(spotPosition);
	}

	if(options->areColoredLightsEnabled()) {	
		float deltaAngle = 2.* M_PI / (float) (options->getNbPlayers());
		
		for(int i=0;i<options->getNbPlayers();i++) {
			float phi = (float) i * deltaAngle + currentTime / 4.;
			float lambda = cos(currentTime/3.) ;
			float distLights = (float) (options->getGridSize()) * GRID_INTERVAL * (1.2 + 0.2 * cos(currentTime/5. + (float) i * deltaAngle)) ;		
			Vector3 lightPosition = Vector3(distLights*cos(lambda)*sin(phi), sin(lambda), distLights*cos(lambda)*cos(phi));
			ligthsPoint[i]->setPosition(lightPosition);
		}
	}
		
	if(scene->getCurrentGUI() == menuIndex) {
		float phi = currentTime/5.;
		float theta = cos(currentTime/3.)/8.;
		float distSpot = 150.;
		Vector3 spotDirection = Vector3(sin(theta)*cos(phi), sin(theta)*sin(phi), -cos(theta));
		Vector3 spotPosition = -distSpot * spotDirection;
		spotMenu->setSpotDirection(spotDirection);
		spotMenu->setPosition(spotPosition);
	}
}


void TicTacToe::initGame() {
	player = 1;
	remainingMoves = grid->getNbSpheres();
	score->reinit();
	reinitPosCamera();
	grid->reinit();	
	
	Color groundColor = Color(options->getColor(1), -0.985, 1.);
	Color skyColor = Color(options->getColor(0), -0.97, 1.);
	scene->getDisplay3D(gameDisplayIndex)->setAmbientColors(groundColor, skyColor, Vector3(0.,1.,0.));
	scene->getDisplay3D(scoreDisplayIndex)->setAmbientColors(groundColor, skyColor, Vector3(0.,1.,0.));	
}



void TicTacToe::quit() {
	options->writeConfigurationFile();
}


bool TicTacToe::keyboardEvent(SDLKey key, double timeInterval) {
	float speed = 2.;

	if(key == generalKeyboardOptions->getKey("quit_key")) {	
		if(scene->getCurrentGUI() < 0) {
			scene->setCurrentGUI(menuIndex);
		} else {
			scene->setCurrentGUI(-1);
			mQuit = true;
		}	
		return false;
	}	

	if(scene->getCurrentGUI() < 0) {
		if(key==mKeyboardOptions->getKey("up_key")) {
			gameCamera->rotAroundO(true, distCam, 0., timeInterval*speed, LIMIT_CAM_TTT, LIMIT_CAM_TTT);	
			return true;
		}
		if(key==mKeyboardOptions->getKey("down_key")) {
			gameCamera->rotAroundO(true, distCam, 0., -timeInterval*speed, LIMIT_CAM_TTT, LIMIT_CAM_TTT);	
			return true;
		}		
		if(key==mKeyboardOptions->getKey("left_key")) {
			gameCamera->rotAroundO(true, distCam, -timeInterval*speed, 0., LIMIT_CAM_TTT, LIMIT_CAM_TTT);	
			return true; 
		}
		if(key==mKeyboardOptions->getKey("right_key")) {
			gameCamera->rotAroundO(true, distCam, timeInterval*speed, 0., LIMIT_CAM_TTT, LIMIT_CAM_TTT);	
			return true; 
		}
		if(key==mKeyboardOptions->getKey("reinit_camera_key")) {
			reinitPosCamera();
			return false;
		}
	}

	
	return false;
}


void TicTacToe::mouseMotion(int xOld, int yOld, int xNew, int yNew, bool left, bool middle, bool right) {
	if((right)&&(scene->getCurrentGUI() < 0)) {
		gameCamera->rotAroundO(true, distCam, STEP_ANGLE*0.2*(xOld - xNew), STEP_ANGLE*0.2*(yNew -yOld), LIMIT_CAM_TTT, LIMIT_CAM_TTT);	
		grid->selectEntityTTT(player, -1);
	} else {
		if(options->isAutoSelectionEnabled()) {
			int hit = scene->getHit(options->getGridSize()*8, xNew, yNew, 4., 4., (float) (options->getGridSize()) * GRID_INTERVAL * 2.);	
			if(scene->getCurrentGUI() < 0) {
				grid->selectEntityTTT(player, hit);
			} else if(scene->getCurrentGUI() == menuIndex) {
				menu->selectMenuItem(hit);
			}
		}
	}
}

void TicTacToe::leftClick(int x, int y) {
	
	if(scene->getCurrentGUI() < 0) {
	
		int hit = scene->getHit(options->getGridSize()*8, x, y, 4., 4., (float) (options->getGridSize()) * GRID_INTERVAL * 2.);	
		
		int nbPlayers = options->getNbPlayers();
		
		if(grid->changePlayerTTT(player, hit)) {
			
			int inc = grid->incScore(player, hit);
			if(inc != 0) {
				score->incScore(player, inc);
				std::cout << "Player " << player << " has now " << score->getScore(player) << " points. \n";				
			}
			
			remainingMoves --;
			player = (player % nbPlayers) + 1;

			int numColor = options->getColor(player);
			
			scene->setClearColor(numColor,-0.95);		
			gameFog->setColor(numColor, -0.95);
			scoreFog->setColor(numColor, -0.95);
			
			Color groundColor = Color(numColor, -0.985, 1.);
			Color skyColor = Color(options->getColor(0), -0.97, 1.);
			scene->getDisplay3D(gameDisplayIndex)->setAmbientColors(groundColor, skyColor, Vector3(0.,1.,0.));	
			scene->getDisplay3D(scoreDisplayIndex)->setAmbientColors(groundColor, skyColor, Vector3(0.,1.,0.));		
			
		}
		
	} else if(scene->getCurrentGUI() == menuIndex) {
		int selectedMenuItem = menu->getSelectedMenuItem();
		
		switch(selectedMenuItem) {
			case 0:
				scene->setCurrentGUI(-1);
				break;
				
			case 1:
				scene->setCurrentGUI(-1);
				initGame();		
				break;
				
			case 2:
				scene->setCurrentGUI(-1);
				mQuit = true;	
				break;
		}
	}
}

void TicTacToe::rightClick(int x, int y) {
	
}

void TicTacToe::middleClick(int x, int y) {
	if(scene->getCurrentGUI() < 0) {
		reinitPosCamera();
	}
}

void TicTacToe::mouseWheel(int w) {
	float newDist = distCam - w * 10.;
	float maxDistCam = (float) (options->getGridSize()) * GRID_INTERVAL * 2.;
	float minDistCam = (float) (options->getGridSize()) * GRID_INTERVAL * 0.8;
	
	
	if(newDist > minDistCam) {
		if(newDist < maxDistCam) {
			distCam = newDist;
		} else {
			distCam = maxDistCam;
		}
	} else {
		distCam = minDistCam;
	}
	
	gameCamera->setDistToO(distCam);
}

/**********************************************************/
/*		TicTacToeCube									  */
/**********************************************************/

TicTacToeCube::TicTacToeCube(): TicTacToe (){	
	init();		
	initGame();
}

TicTacToeCube::~TicTacToeCube() {
}

void TicTacToeCube::createObjects () {
	Display3D *gameDisp3D = scene->getDisplay3D(gameDisplayIndex);
	
	grid = new GridCubeTTT(options->getGridSize(), options->getColors(), SPHERES_RADIUS, CYLINDER_RADIUS, GRID_INTERVAL);
	grid->setNumClick(-1);
	gameDisp3D->addEntity(grid); 

	
	createScore();	
	createMenu();
}


void TicTacToeCube::reinitPosCamera() {
	distCam = distInitCam;
	Vector3 pos = Vector3(0.,0.,distCam);
	gameCamera->setPositionLookingAtO(pos);
	gameCamera->rotAroundO(true, distCam, -0.4, 0.4, LIMIT_CAM_TTT, LIMIT_CAM_TTT);
}




