/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "roundEdge.h"
#include "../newtEngine/scene.h"


RoundEdgeMesh::RoundEdgeMesh(float height, int nbH, float HTexBegin, float HTexEnd, float radius, float angle, int nbR, float RTexBegin, float RTexEnd, bool inside) :
	Mesh("roundEdge", false, true, false, false), 
	mHeight(height), mHTexBegin(HTexBegin), mHTexEnd(HTexEnd),
	mRadius(radius), mAngle(angle), mRTexBegin(RTexBegin), mRTexEnd(RTexEnd), mInside(inside) {
					
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;
		
	value = (float) nbH * meshLevel;
	if(value < 1) {
		mNbH = 1;
	} else {
		mNbH = (int) value;
	}

	value = (float) nbR * meshLevel;
	if(value < 2) {
		mNbR = 2;
	} else {
		mNbR = (int) value;
	}				
}

RoundEdgeMesh::~RoundEdgeMesh() {
}
	
void RoundEdgeMesh::initFirstParams(){
	
	geometry.nbVertices = (mNbH + 1) * (mNbR + 1);
	
	geometry.nbIndexArrays = mNbR;
	geometry.primType = new GLenum[mNbR];
	for(int i=0;i<mNbR;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	
	geometry.nbIndices = new int[mNbR];
	for(int i=0;i<mNbR;i++) {
		geometry.nbIndices[i] = 2 * (mNbH+1);
	}		
	
	mpUniformTangent = new Vector3(1., 0., 0.);	
}
		
void RoundEdgeMesh::createVertices(Vector3 *ptr){
	float alpha;
	
	for(int i=0;i<=mNbR;i++) {
		alpha = ((float) i/mNbR  - 1./2.)*mAngle; 
		for(int j=0;j<=mNbH;j++) {
			*ptr = Vector3(((float) (j/mNbH) - 1./2.) * mHeight, mRadius * sin(alpha), mRadius * cos(alpha));
	    	ptr ++;
		}
	}	
}

void RoundEdgeMesh::createColors(Color *ptr){
	
}

void RoundEdgeMesh::createNormals(Vector3 *ptr) {
	float alpha;
		
	if(mInside) {
		for(int i=0;i<=mNbR;i++) {
			alpha = ((float) i/mNbR  - 1./2.)*mAngle; 
			for(int j=0;j<=mNbH;j++) {
				*ptr = Vector3(0., -sin(alpha), -cos(alpha));
				ptr ++;
			}
		}
	} else {
		for(int i=0;i<=mNbR;i++) {
			alpha = ((float) i/mNbR  - 1./2.)*mAngle; 
			for(int j=0;j<=mNbH;j++) {
				*ptr = Vector3(0., sin(alpha), cos(alpha));
				ptr ++;
			}
		}				
	}
}

void RoundEdgeMesh::createTexCoords(GLfloat *ptr) {
	for(int i=0;i<mNbR+1;i++){
		for(int j=0;j<mNbH+1;j++){
			*ptr = mHTexBegin + (GLfloat) j/mNbH * (mHTexEnd - mHTexBegin);
	    	ptr ++;
			*ptr = mRTexBegin + (GLfloat) i/mNbR * (mRTexBegin - mRTexEnd);
	    	ptr ++;		
		}
	}
}

void RoundEdgeMesh::createFogCoords(GLfloat *ptr) {

}
	
void RoundEdgeMesh::createTangents(Vector3 *ptr) {
	//uniform tangent
}
	

void RoundEdgeMesh::createIndices(int numArray, GLuint *ptr) {

	if(mInside) {
		for(int j=0;j<=mNbH;j++){
			*ptr = numArray*(mNbH+1) + j;
			ptr++;				
			*ptr = (numArray +1)*(mNbH+1) + j;	
			ptr++;		
		}
	} else {
		for(int j=0;j<=mNbH;j++){
			*ptr = (numArray +1)*(mNbH+1) + j;	
			ptr++;
			*ptr = numArray*(mNbH+1) + j;
			ptr++;
		}
	}
}


